/*	$Id: debug.c,v 1.15 2005/08/22 21:48:34 phil Exp $ */

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 1987-2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/


//
//  Coda debugging routines / pjb, fall 1998
//
//  Modified by Phil Nelson, Summer 2001.
//

// Do nothing for this in the free version.
#ifdef DBG

#include <stdarg.h>
#include <ntddk.h>
#include <coda_types.h>
#include <coda.h>
#include <coda_nt.h>
#include <snprintf.h>

// Set the levels here!

ULONG DebugLevel =
//	D_SUPER    |
//	D_INODE    |
//	D_FILE     |
//	D_CACHE    |
//	D_MALLOC   |
//	D_CNODE    |
//	D_UPCALL   |
//	D_IPC      |
//	D_PIOCTL   |
//	D_UID      |
//	D_WRAPPER  |
//	D_LOOKATTR |
//	D_DOWNCALL |
//	D_MISC     |
//	D_ALL	   |
	0;


//
// Global data - this code is NOT intended to be re-entrant!
//

/* Drivername appearing under filesystems */
char * Coda_DriverName = "CodaFSD";
static PSTR OSR_DriverDate = __DATE__;
static PSTR OSR_DriverTime = __TIME__;

// Number of fid buffers.
#define N 3

char *coda_f2s(CodaFid *fid) 
{
	static char fids[N][60];
	static ix = -1;
	ix = (ix + 1) % N; 
	snprintf(fids[ix], 60, "[0x%x 0x%x 0x%x 0x%x]", FID(fid));
	return fids[ix];
}

//
// Debug message storage for user to get them!
//

typedef struct entry {
    char *         e_mesg;
    int            e_len;
    struct entry * e_next;
} dbgentry;

static dbgentry *d_head = NULL, *d_tail = NULL;

static KSEMAPHORE msg_semaphore;
static KSEMAPHORE cr_sem;

void init_msg_list ()
{
    KeInitializeSemaphore (&msg_semaphore, 0, 10000);
    KeInitializeSemaphore (&cr_sem, 1, 1);
}

void clean_msg_list()
{
    dbgentry *t;
    while (d_head != NULL) {
	t = d_head;
	d_head = t->e_next;
	coda_free (t->e_mesg);
	coda_free (t);
    }
    d_tail = NULL;
}

void save_debug_mesg (char *mesg)
{
    dbgentry *t;
    int len;

    // Allocate entry
    t = coda_malloc(sizeof(dbgentry));
    if (!t)
	return;

    // Allocate string storage
    len = strlen(mesg) + 1;
    t->e_mesg = coda_malloc (len);
    if (!t->e_mesg) {
	coda_free (t);
	return;
    }

    // Copy and set up entry
    strcpy (t->e_mesg, mesg);
    t->e_len = len;
    t->e_next = NULL;

    // Add it to the queue.
    (void) KeWaitForSingleObject (&cr_sem, Executive, KernelMode, FALSE, 0);
    if (d_tail != NULL)
	d_tail->e_next = t;
    else
	d_head = t;
    d_tail = t;
    KeReleaseSemaphore (&cr_sem, 0, 1, 0);

    // Signal that message is ready.  Increment by one.
    KeReleaseSemaphore (&msg_semaphore, 0, 1, 0);
}

int get_debug_mesg(char *buffer, int buffersize)
{
    NTSTATUS rc;
    LARGE_INTEGER timeout;
    dbgentry *t;
    int       outsize;

    // Set timeout to about 5 seconds.
    timeout = RtlConvertLongToLargeInteger(-5L * 1000000L * 10L ); 

    // Wait for a message
    rc = KeWaitForSingleObject (&msg_semaphore, Executive, KernelMode,
				TRUE, &timeout);
    if (rc == STATUS_TIMEOUT) {
	return 0;
    }

    // We now have a message!
    (void) KeWaitForSingleObject (&cr_sem, Executive, KernelMode, FALSE, 0);
    t = d_head;
    d_head = t->e_next;
    if (d_head == NULL)
	d_tail = NULL;
    KeReleaseSemaphore (&cr_sem, 0, 1, 0);

    outsize = MIN(t->e_len, buffersize);
    RtlCopyMemory (buffer, t->e_mesg, outsize);

    coda_free (t->e_mesg);
    coda_free (t);

    return outsize;
}


void coda_debug(unsigned int mask, char *fmt, ...)
{
	va_list args;
	int i;
	char msg[160];
	
	// Allow for multiple things to be on before printing!

	if ( mask != (mask & DebugLevel) )
		return;
	
	va_start(args, fmt);
	i=vsnprintf(msg,160,fmt,args);
	va_end(args);
	DbgPrint(msg);

	if (D_GETMSG & DebugLevel)
	    save_debug_mesg(msg);
}

#endif
