// utility to mount and unmount coda
//
//  umount [-m] [-d drive-letter]
//
//  $Id: umount.c,v 1.4 2005/09/13 19:31:01 phil Exp $

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/

#include <windows.h>
#include <stdio.h>
#include <winioctl.h>
#include <string.h>
#include <ctype.h>
#include <wchar.h>
#include <w32api/dbt.h>

#include "../coda/wrapper-ioctl.h"


void myperror(char *str)
{
    DWORD err = GetLastError();
    char buf[1000];
    
    FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM, NULL, err, 0, buf, sizeof(buf),
		   NULL);
    printf ("%s: %s\n", str, buf);
    
}

#define BUFSIZE 4096
int main (int argc, char *argv[])
{
    HANDLE h;
    OW_PSEUDO_MOUNT_INFO info;
    LONG nBytesReturned;
    CHAR outBuf[BUFSIZE];
    DWORD d;
    int n;
    int ctlcode = OW_FSCTL_DISMOUNT_PSEUDO;
    int rv = 0;
    char drive = 'N';
    int mount = 0;
    
    WCHAR link[20] = L"\\??\\N:";  
    
    if (argc > 1) {
	n = 1;
	while (n < argc) {
	    if (strcmp(argv[n],"-m") == 0 ) {
		ctlcode = OW_FSCTL_MOUNT_PSEUDO;
		mount = 1;
		n++;
	    } else if ((strcmp(argv[n],"-d")==0) && (n+1) < argc
		       && strlen(argv[n+1]) == 1
		       && isalpha(argv[n+1][0])) {
	        drive = argv[n+1][0];
		link[4] = (short) drive;
		n+=2;
	    } else {
		printf ("usage: %s [-m] [-d drive-letter]\n",
			argv[0]);
		exit(1);
	    }
	}
    }
    
    d = DefineDosDevice(DDD_RAW_TARGET_PATH, "codadev", "\\Device\\codadev");
    if ( d == 0 ) 
	myperror("DDD failed");
    h = CreateFile ("\\\\.\\codadev", GENERIC_READ | GENERIC_WRITE,
		    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
		    OPEN_EXISTING, 0, NULL);
    if (h == INVALID_HANDLE_VALUE) {
	myperror ("CreateFile failed");
    } else {
	printf ("CreateFile returned %d\n", (int)h);
    }
    
    // Set up the info for the DeviceIoControl.
    info.PseudoVolumeHandle = (HANDLE*)1;
    info.PseudoDeviceName = L"\\Device\\coda";
    info.PseudoDeviceNameLength =
	wcslen(info.PseudoDeviceName) * sizeof(WCHAR);
    info.PseudoLinkName = link;
    info.PseudoLinkNameLength = wcslen(info.PseudoLinkName) * sizeof(WCHAR);
    
    if ((d = DeviceIoControl(h, ctlcode, &info, sizeof(info), NULL, 0,
			     &nBytesReturned, NULL))) {
	printf ("request succeeded: (%ld) %s\n", nBytesReturned, outBuf);
    } else {
	myperror ("request failed\n");
	rv = 1;
    }
 
    {  /* Let others know */

        static DEV_BROADCAST_VOLUME DevBcst;
        DWORD BcstFlag;
	WPARAM ChKind;

        DevBcst.dbcv_devicetype = DBT_DEVTYP_VOLUME;
	DevBcst.dbcv_size = sizeof(DEV_BROADCAST_VOLUME);
	DevBcst.dbcv_flags = DBTF_NET;
	DevBcst.dbcv_unitmask =  1 << (toupper(drive) - 'A');

	if (mount)
	    ChKind = DBT_DEVICEARRIVAL;
	else
	    ChKind = DBT_DEVICEREMOVECOMPLETE;

	BcstFlag = BSM_ALLCOMPONENTS | BSM_ALLDESKTOPS;
	BroadcastSystemMessage (BSF_IGNORECURRENTTASK, &BcstFlag,
				WM_DEVICECHANGE, ChKind, 
				(LPARAM) &DevBcst);
   
    }
    return rv;
}

